@@ -1,10 +1,16 @@
-import functools
-import logging
-import threading
 import time
+import logging
+import functools
+import threading
+from six import raise_from
+
+try:
+    from json.decoder import JSONDecodeError
+except ImportError:
+    JSONDecodeError = ValueError
 
-import requests
 import six
+import requests
 
 from sevenbridges.errors import (
     BadRequest, Unauthorized, Forbidden, NotFound, MethodNotAllowed,
@@ -83,7 +89,7 @@ def retry(retry_count):
             for backoff in range(retry_count):
                 try:
                     return f(*args, **kwargs)
-                except:
+                except Exception:
                     time.sleep(2 ** backoff)
             else:
                 raise SbgError('{}: failed to complete: {}'.format(
@@ -95,6 +101,21 @@ def retry(retry_count):
     return func
 
 
+def throttle(func):
+    """Throttles number of parallel requests made by threads from single
+    HttpClient session."""
+
+    # noinspection PyProtectedMember
+    @functools.wraps(func)
+    def wrapper(http_client, *args, **kwargs):
+        if http_client._throttle_limit:
+            with http_client._throttle_limit:
+                return func(http_client, *args, **kwargs)
+        else:
+            return func(http_client, *args, **kwargs)
+    return wrapper
+
+
 def check_for_error(func):
     """
     Executes the wrapped function and inspects the response object
@@ -134,6 +155,14 @@ def check_for_error(func):
             raise e
         except requests.RequestException as e:
             raise SbgError(message=six.text_type(e))
+        except JSONDecodeError:
+            message = (
+                'Service might be unavailable. Can also occur by providing '
+                'too many query parameters.'
+            )
+            raise_from(
+                ServiceUnavailable(message=six.text_type(message)), None
+            )
         except ValueError as e:
             raise SbgError(message=six.text_type(e))
 
