@@ -1,14 +1,16 @@
+import copy
 import json
 import logging
 import platform
+import threading
 from datetime import datetime as dt
 
 import requests
 
 import sevenbridges
+from sevenbridges.decorators import check_for_error, throttle
+from sevenbridges.errors import SbgError, URITooLong
 from sevenbridges.config import Config, format_proxies
-from sevenbridges.decorators import check_for_error
-from sevenbridges.errors import SbgError
 from sevenbridges.http.error_handlers import maintenance_sleeper
 
 logger = logging.getLogger(__name__)
@@ -26,13 +28,47 @@ class AAHeader:
     value = 'Advance'
 
 
-def generate_session(proxies=None):
+class RequestSession(requests.Session):
+    """Client session class"""
+
+    MAX_URL_LENGTH = 6000
+
+    def send(self, request, **kwargs):
+        """Send prepared request
+        :param request: Prepared request to be sent
+        :param kwargs: request keyword arguments
+        :return: Request response
+        """
+        if len(request.url) > self.MAX_URL_LENGTH:
+            raise URITooLong(
+                message=(
+                    'Request url too large, '
+                    'likely too many query parameters provided.'
+                )
+            )
+        return super(RequestSession, self).send(request, **kwargs)
+
+
+def generate_session(pool_connections, pool_maxsize, pool_block, proxies=None):
     """
     Utility method to generate request sessions.
+    :param pool_connections: The number of urllib3 connection pools to
+        cache.
+    :param pool_maxsize: The maximum number of connections to save in the
+        pool.
+    :param pool_block: Whether the connection pool should block for
+        connections.
     :param proxies: Proxies dictionary.
     :return: requests.Session object.
     """
-    session = requests.Session()
+    session = RequestSession()
+    adapter = requests.adapters.HTTPAdapter(
+        pool_connections=pool_connections,
+        pool_maxsize=pool_maxsize,
+        pool_block=pool_block
+    )
+    session.mount('http://', adapter)
+    session.mount('https://', adapter)
     session.proxies = proxies
     return session
 
@@ -53,11 +89,17 @@ def config_vars(profiles, advance_access):
             proxies = config.proxies
             aa = config.advance_access
             return url, token, proxies, aa
-        except:
+        except Exception:
             pass
     return None, None, None, None
 
 
+def mask_secrets(request_data):
+    masked = copy.deepcopy(request_data)
+    masked['headers']['X-SBG-Auth-Token'] = '*****'
+    return masked
+
+
 # noinspection PyTypeChecker
 class HttpClient(object):
     """
@@ -67,7 +109,9 @@ class HttpClient(object):
 
     def __init__(self, url=None, token=None, oauth_token=None, config=None,
                  timeout=None, proxies=None, error_handlers=None,
-                 advance_access=False):
+                 advance_access=False, pool_connections=None,
+                 pool_maxsize=None, pool_block=True,
+                 max_parallel_requests=None):
 
         if (url, token, config) == (None, None, None):
             url, token, proxies, advance_access = config_vars(
@@ -88,13 +132,27 @@ class HttpClient(object):
             advance_access = advance_access
 
         if not url:
-            raise SbgError('URL is missing!'
-                           ' Configuration may contain errors, '
-                           'or you forgot to pass the url param.')
+            raise SbgError(
+                'URL is missing! Configuration may contain errors, '
+                'or the url parameter is missing.'
+            )
 
         self.url = url.rstrip('/')
-        self._session = generate_session(proxies)
+        self.pool_connections = pool_connections
+        self.pool_maxsize = pool_maxsize
+        self.pool_block = pool_block
+        self._session = generate_session(
+            pool_connections,
+            pool_maxsize,
+            pool_block,
+            proxies=proxies
+        )
         self.timeout = timeout
+        self._throttle_limit = (
+            threading.Semaphore(max_parallel_requests)
+            if max_parallel_requests
+            else None
+        )
         self._limit = None
         self._remaining = None
         self._reset = None
@@ -113,15 +171,17 @@ class HttpClient(object):
         elif self.oauth_token:
             self.headers['Authorization'] = 'Bearer {}'.format(oauth_token)
         else:
-            raise SbgError('Required authorization model not selected!. '
-                           'Please provide at least one token value.'
-                           )
+            raise SbgError(
+                'Required authorization model not selected!. '
+                'Please provide at least one token value.'
+            )
 
         self.aa = advance_access
         if self.aa:
-            logger.warning('Advance access features enabled. '
-                           'AA API calls can be subjected to changes'
-                           )
+            logger.warning(
+                'Advance access features enabled. '
+                'AA API calls can be subject to changes.'
+            )
         self.error_handlers = [maintenance_sleeper]
         if error_handlers and isinstance(error_handlers, list):
             for handler in error_handlers:
@@ -164,6 +224,7 @@ class HttpClient(object):
     def _rate_limit(self):
         self._request('GET', url='/rate_limit', append_base=True)
 
+    @throttle
     @check_for_error
     def _request(self, verb, url, headers=None, params=None, data=None,
                  append_base=False, stream=False):
@@ -173,7 +234,7 @@ class HttpClient(object):
             headers = self.headers
         else:
             headers.update(self.headers)
-        if not self.token or self.oauth_token:
+        if not (self.token or self.oauth_token):
             raise SbgError(message='Api instance must be authenticated.')
 
         if hasattr(self, '_session_id'):
@@ -187,16 +248,28 @@ class HttpClient(object):
         if self.aa:
             self.headers[AAHeader.key] = AAHeader.value
 
-        d = {'verb': verb, 'url': url, 'headers': headers, 'params': params}
+        request_data = {
+            'verb': verb,
+            'url': url,
+            'headers': headers,
+            'params': params
+        }
+        masked_request_data = mask_secrets(request_data)
         if not stream:
-            d.update({'data': data})
-            logger.debug("Request %s", str(d), extra=d)
+            masked_request_data.update({'data': data})
+            logger.debug(
+                "Request %s", masked_request_data,
+                extra=masked_request_data
+            )
             response = self._session.request(
                 verb, url, params=params, data=json.dumps(data),
                 headers=headers, timeout=self.timeout, stream=stream
             )
         else:
-            logger.debug('Stream Request', extra=d)
+            logger.debug(
+                'Stream Request %s', masked_request_data,
+                extra=masked_request_data
+            )
             response = self._session.request(
                 verb, url, params=params, stream=stream, allow_redirects=True,
             )
